
module bts_config (    
  input  wire          clk_fpga_p   ,    // system clock, 100M
  input  wire          fpga_resetn  ,  

//I2C interfaces
  inout  wire		   fmc_scl , 
  inout  wire		   fmc_sda , 
  inout  wire		   cxl_scl , 
  inout  wire		   cxl_sda ,    
  inout  wire		   qsfp_scl , 
  inout  wire		   qsfp_sda , 
  inout  wire		   rdimm_i2c_scl , 
  inout  wire		   rdimm_i2c_sda , 
  
//spi
  output  wire		   si5518_sclk , 
  output  wire		   si5518_cs_n ,
  output  wire		   si5518_mosi ,
  input  wire          si5518_miso  ,
  
//qsfp control signals
  output wire          qsfpdd800_reset_n,
  output wire          qsfpdd800_modsel_n,
  input  wire          qsfpdd800_modprs_n,
  output wire          qsfpdd800_lpmode,
  input  wire          qsfpdd800_int_n,
  
  output wire          qsfpdd0_reset_n,
  output wire          qsfpdd0_modsel_n,
  input  wire          qsfpdd0_modprs_n,
  output wire          qsfpdd0_lpmode,
  input  wire          qsfpdd0_int_n,
  
  output wire          qsfpdd1_reset_n,
  output wire          qsfpdd1_modsel_n,
  input  wire          qsfpdd1_modprs_n,
  output wire          qsfpdd1_lpmode,
  input  wire          qsfpdd1_int_n,

//LED   
  output wire          fpga_led0	, 		//High: LED on						
  output wire          fpga_led1    ,
  output wire          fpga_led2	,
  output wire          fpga_led3    ,

  output wire          qsfpdd0_led0	, 		//High: LED on						
  output wire          qsfpdd0_led1 ,    
  output wire          qsfpdd1_led0	, 		//High: LED on						
  output wire          qsfpdd1_led1 , 
  output wire          qsfpdd800_led0 , 		//High: LED on						
  output wire          qsfpdd800_led1 , 

//other
  output wire          si5518_rstn ,
  input wire           si5392_lolb ,
  output wire          sdi_clk148_5m_up,
  output wire          sdi_clk148_5m_down
);

//--------------------------------------------------------------------------------------------------------------------------
wire          ninit_done     ;	
wire          S_global_resetn;	  
wire          clk_50m        ;

wire          fmc_scl_in;
wire          fmc_sda_in;
wire          fmc_scl_oe;
wire          fmc_sda_oe;   

wire          cxl_scl_in;
wire          cxl_sda_in;
wire          cxl_scl_oe;
wire          cxl_sda_oe;


wire          qsfp_scl_in;
wire          qsfp_sda_in;
wire          qsfp_scl_oe;
wire          qsfp_sda_oe;

wire          rdimm_i2c_scl_in;
wire          rdimm_i2c_sda_in;
wire          rdimm_i2c_scl_oe;
wire          rdimm_i2c_sda_oe;

wire [3:0]    fpga_led; 
wire [5:0]    module_led;     

wire [9:0]    qsfp_ctrl;    

wire [6:0]    S_pio_status;    
   
assign S_global_resetn = !ninit_done && fpga_resetn; //NINIT_DONE is high until the entire device enters user mode
assign S_pio_status = {qsfpdd800_int_n, qsfpdd1_int_n, qsfpdd0_int_n, si5392_lolb, qsfpdd800_modprs_n, qsfpdd1_modprs_n, qsfpdd0_modprs_n};
assign fpga_led3 = fpga_led[3];
assign fpga_led2 = fpga_led[2];
assign fpga_led1 = fpga_led[1];
assign fpga_led0 = fpga_led[0];

assign qsfpdd0_led0 = module_led[0];
assign qsfpdd0_led1 = module_led[1];
assign qsfpdd1_led0 = module_led[2];
assign qsfpdd1_led1 = module_led[3];
assign qsfpdd800_led0 = module_led[4];
assign qsfpdd800_led1 = module_led[5];

assign qsfpdd800_reset_n = qsfp_ctrl[0];
assign qsfpdd800_modsel_n = qsfp_ctrl[1];
assign qsfpdd0_reset_n = qsfp_ctrl[2];
assign qsfpdd0_modsel_n = qsfp_ctrl[3];
assign qsfpdd1_reset_n = qsfp_ctrl[4];
assign qsfpdd1_modsel_n = qsfp_ctrl[5];
assign qsfpdd0_lpmode = qsfp_ctrl[6];
assign qsfpdd1_lpmode = qsfp_ctrl[7];
assign qsfpdd800_lpmode = qsfp_ctrl[8];
assign si5518_rstn = qsfp_ctrl[9];
assign sdi_clk148_5m_up = 1'bZ;
assign sdi_clk148_5m_down = 1'bZ;

  
  reset_release reset_release (
	.ninit_done(ninit_done)
	);
 
  sys_pll sys_pll_inst(
  	.rst      (!S_global_resetn), 
  	.refclk   (clk_fpga_p      ),
  	.locked   (  ),
  	.outclk_0 (clk_50m         )
  	); 
 
  q_sys q_sys_inst(                                                               
		.clk_clk                                     (clk_50m               ),                                     //   input,   width = 1,                                  clk.clk
		.reset_reset_n                               (S_global_resetn       ),                               //   input,   width = 1,                                reset.reset_n
		.i2c_0_i2c_serial_sda_in                     (fmc_sda_in       ),                     //   input,   width = 1,                     i2c_0_i2c_serial.sda_in
		.i2c_0_i2c_serial_scl_in                     (fmc_scl_in       ),                     //   input,   width = 1,                                     .scl_in
		.i2c_0_i2c_serial_sda_oe                     (fmc_sda_oe       ),                     //  output,   width = 1,                                     .sda_oe
		.i2c_0_i2c_serial_scl_oe                     (fmc_scl_oe       ),                     //  output,   width = 1,                                     .scl_oe
        .i2c_1_i2c_serial_sda_in                  (cxl_sda_in),                  //   input,  width = 1,                  i2c_1_i2c_serial.sda_in
		.i2c_1_i2c_serial_scl_in                  (cxl_scl_in),                  //   input,  width = 1,                                  .scl_in
		.i2c_1_i2c_serial_sda_oe                  (cxl_sda_oe),                  //  output,  width = 1,                                  .sda_oe
		.i2c_1_i2c_serial_scl_oe                  (cxl_scl_oe),                  //  output,  width = 1,                                  .scl_oe
		//.i2c_2_i2c_serial_sda_in                  (si5518_sda_in),                  //   input,  width = 1,                  i2c_2_i2c_serial.sda_in
		//.i2c_2_i2c_serial_scl_in                  (si5518_scl_in),                  //   input,  width = 1,                                  .scl_in
		//.i2c_2_i2c_serial_sda_oe                  (si5518_sda_oe),                  //  output,  width = 1,                                  .sda_oe
		//.i2c_2_i2c_serial_scl_oe                  (si5518_scl_oe),                  //  output,  width = 1,                                  .scl_oe
		.i2c_3_i2c_serial_sda_in                  (qsfp_sda_in),                  //   input,  width = 1,                  i2c_3_i2c_serial.sda_in
		.i2c_3_i2c_serial_scl_in                  (qsfp_scl_in),                  //   input,  width = 1,                                  .scl_in
		.i2c_3_i2c_serial_sda_oe                  (qsfp_sda_oe),                  //  output,  width = 1,                                  .sda_oe
		.i2c_3_i2c_serial_scl_oe                  (qsfp_scl_oe),                  //  output,  width = 1,                                  .scl_oe
		.i2c_4_i2c_serial_sda_in                  (rdimm_i2c_sda_in),                  //   input,  width = 1,                  i2c_4_i2c_serial.sda_in
		.i2c_4_i2c_serial_scl_in                  (rdimm_i2c_scl_in),                  //   input,  width = 1,                                  .scl_in
		.i2c_4_i2c_serial_sda_oe                  (rdimm_i2c_sda_oe),                  //  output,  width = 1,                                  .sda_oe
		.i2c_4_i2c_serial_scl_oe                  (rdimm_i2c_scl_oe),                  //  output,  width = 1,                                  .scl_oe
		.pio_status_external_connection_export    (S_pio_status), 
		.pio_led_external_connection_export       (fpga_led),       //  output,  width = 4,       pio_led_external_connection.export
		.module_led_external_connection_export    (module_led),
		.pio_qsfp_ctrl_external_connection_export (qsfp_ctrl), //  output,  width = 10, pio_qsfp_ctrl_external_connection.export
	    .spi_0_external_MISO                      (si5518_miso),                      //   input,  width = 1,                    spi_0_external.MISO
		.spi_0_external_MOSI                      (si5518_mosi),                      //  output,  width = 1,                                  .MOSI
		.spi_0_external_SCLK                      (si5518_sclk),                      //  output,  width = 1,                                  .SCLK
		.spi_0_external_SS_n                      (si5518_cs_n)                       //  output,  width = 1,                                  .SS_n
	);	                                                                    

assign fmc_scl_in             = fmc_scl;
assign fmc_sda_in             = fmc_sda;
assign fmc_scl                = fmc_scl_oe ? 1'b0 : 1'bz;
assign fmc_sda                = fmc_sda_oe ? 1'b0 : 1'bz;	

assign cxl_scl_in             = cxl_scl;
assign cxl_sda_in             = cxl_sda;
assign cxl_scl                = cxl_scl_oe ? 1'b0 : 1'bz;
assign cxl_sda                = cxl_sda_oe ? 1'b0 : 1'bz;

assign qsfp_scl_in             = qsfp_scl;
assign qsfp_sda_in             = qsfp_sda;
assign qsfp_scl                = qsfp_scl_oe ? 1'b0 : 1'bz;
assign qsfp_sda                = qsfp_sda_oe ? 1'b0 : 1'bz;

assign rdimm_i2c_scl_in             = rdimm_i2c_scl;
assign rdimm_i2c_sda_in             = rdimm_i2c_sda;
assign rdimm_i2c_scl                = rdimm_i2c_scl_oe ? 1'b0 : 1'bz;
assign rdimm_i2c_sda                = rdimm_i2c_sda_oe ? 1'b0 : 1'bz;							     

endmodule	
	