// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



module ed_synth_perf_monch1_hydra_reset_controller_100_si24uqa # (
   parameter IS_SIMULATION       = 1,
   parameter NUM_RESET_INPUTS    = 1,
   parameter NUM_RESET_OUTPUTS   = 1
) (
   input  logic in_reset_n_0,

   output logic out_reset_n_0,
   output logic out_reset_n_1,
   output logic out_reset_n_2,
   output logic out_reset_n_3,

   input  logic clk,

   input  logic locked
);
   initial begin
      assert(NUM_RESET_INPUTS  == 1)
         else $fatal(1, "IP was configured for NUM_RESET_INPUTS=%0d, but RTL has NUM_RESET_INPUTS=%0d", 1, NUM_RESET_INPUTS);
      assert(NUM_RESET_OUTPUTS == 4)
         else $fatal(1, "IP was configured for NUM_RESET_OUTPUTS=%0d, but RTL has NUM_RESET_OUTPUTS=%0d", 4, NUM_RESET_OUTPUTS);
   end

   logic [NUM_RESET_INPUTS-1:0]  in_reset_n;
   logic [NUM_RESET_OUTPUTS-1:0] out_reset_n;

   always_comb begin
      in_reset_n[0] = in_reset_n_0;

      out_reset_n_0 = out_reset_n[0];
      out_reset_n_1 = out_reset_n[1];
      out_reset_n_2 = out_reset_n[2];
      out_reset_n_3 = out_reset_n[3];
   end

   logic issp_reset_n;
   if (!IS_SIMULATION) begin : issp
      altsource_probe #(
         .sld_auto_instance_index ("YES"),
         .sld_instance_index      (0),
         .instance_id             ("HRST"),
         .probe_width             (0),
         .source_width            (1),
         .source_initial_value    ("1"),
         .enable_metastability    ("NO") // ("YES")
      ) reset_issp_inst (
         .source        (issp_reset_n)
      );
   end else begin : no_issp
      assign issp_reset_n = '1;
   end

   hydra_reset_controller_impl # (
      .NUM_RESET_INPUTS    (NUM_RESET_INPUTS + 2),
      .NUM_RESET_OUTPUTS   (NUM_RESET_OUTPUTS)
   ) reset_ctrl_impl (
      .clk           (clk),
      .in_reset_n    ({in_reset_n, issp_reset_n, locked}),
      .out_reset_n   (out_reset_n)
   );

endmodule



